#!/usr/bin/perl
#----------------------------------------------------------------------
# Modules and pragmas
# ----------------------------------------------------------------------
use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/perl5";

use Getopt::Long;

use SDL;
use SDL::App;
use SDL::Surface;
use SDL::Cursor;
use SDL::Event;
use SDL::Color;
use SDL::TTFont;
use SDL::Rect;
# ----------------------------------------------------------------------
# Global variables
# ----------------------------------------------------------------------
our $APP;
our $ACTIONS        = {};
our %FILES          = ();
our %SIZE           = ();
our %BACKGROUND     = ();
our %TITLE          = ();
our %MENU           = ();
our %INFO           = ();
our %BACK           = ();
our %FONTS          = ();
our @CATEGORIES     = ();
our @GAMES          = ();
# ----------------------------------------------------------------------
# 
# ----------------------------------------------------------------------
my $fileconf   = "lihuenedu.cfg";
my $name       = "lihuenedu";
my $version    = "0.2-hack";
my $bpp        = 24;  # alterar en linea de comandos
my $fullscreen = 0;
my $keyrepeat  = 1;
# ----------------------------------------------------------------------
#
# ----------------------------------------------------------------------
my $category;
my $widget;
# ----------------------------------------------------------------------
# Parse command line 
# ----------------------------------------------------------------------
{
    my %options; 
    GetOptions(\%options, "bpp=i", "fullscreen",
                          "nokeyrepeat", "help" );

    if ($options{help}) {
	print <<EOF;
$name by SoporteLihuen (soportelihuen at mail.linti.unlp.edu.ar)
Based on HQgames by Pablo Santamaria (pablosantamaria at gmail.com)

    Syntax: $name [options] 

    Options:
        --bpp=8|16|24|32    Sets the colour depth  (default 24)
        --fullscreen        Use full-screen display (default no)
	--nokeyrepeat       Disable keyboard key repeat (default enable)
	--help              This help


    KEYS
	These are the default key assignments:

	Up Arrow    Previous list item
	Down Arrow  Next list item
	Return      Press selected list item
	Backspace   
        Escape/q    Quit
	f           Toggle fullscreen           

EOF
	exit(0);
    }

    $fullscreen = 1     if $options{fullscreen};
    $keyrepeat  = 0     if $options{nokeyrepeat};
    if ($options{bpp}) {
	unless ($options{bpp} == 8  or 
	        $options{bpp} == 16 or
		$options{bpp} == 24 or
		$options{bpp} == 32 ) {
	    die "bpp not valid.\n";
	}
	$bpp = $options{bpp};
    }
}
# -------------------------------------------------------------
# Load configuration file: fill %SIZE and %FILES
# -------------------------------------------------------------
die "Config file: \"$fileconf\" not found.\n" unless -f $fileconf;    
do $fileconf;
# -------------------------------------------------------------
# Create the main surface
# -------------------------------------------------------------
$APP = new SDL::App (  
    -width  => $SIZE{width},
    -height => $SIZE{height},
    -depth  => $bpp,
    -title  => $name,
    -flags  => ($fullscreen ? SDL_FULLSCREEN : 0)| SDL_DOUBLEBUF | SDL_HWSURFACE | SDL_HWACCEL,
);
# -------------------------------------------------------------
# Define actions for pages 
# -------------------------------------------------------------
my $actions_page_intro = {
    SDL_MOUSEMOTION()     => \&process_mousemotion_page_intro,
    SDL_MOUSEBUTTONDOWN() => \&process_mousebuttondown_page_intro,
    SDL_KEYDOWN()         => \&process_keydown_page_intro,
};    

my $actions_page_games = {
    SDL_MOUSEMOTION()     => \&process_mousemotion_page_games,
    SDL_MOUSEBUTTONDOWN() => \&process_mousebuttondown_page_games,
    SDL_KEYDOWN()         => \&process_keydown_page_games,
};    
# -------------------------------------------------------------
# Draw the main page
# -------------------------------------------------------------
draw_page_main();
# -------------------------------------------------------------
# Add the event loop.
# -------------------------------------------------------------
my $event = new SDL::Event;

# /usr/include/SDL/SDL_keyboard.h
# SDL_DEFAULT_REPEAT_DELAY    = 500
# SDL_DEFAULT_REPEAT_INTERVAL = 30
$event -> set_key_repeat(500,100) if $keyrepeat;

MAIN_LOOP:
while(1) {

    while ($event->poll) {
	my $type = $event->type();

	last MAIN_LOOP if ($type == SDL_QUIT);

	if ( $type == SDL_KEYDOWN ) {
	    my $key = $event->key_name();
	    foreach ($key) {
		$_ eq 'f'      && do { $APP-> fullscreen ; last };
		$_ eq 'escape' && do { last MAIN_LOOP };
		$_ eq 'q'      && do { last MAIN_LOOP };
	    }
	}

	if ( exists($ACTIONS->{$type}) && ref( $ACTIONS->{$type} ) eq "CODE" ) {
	    $ACTIONS->{$type}->($event);
	}	    

    }
    
    $APP->delay(5);
}

exit(0);  
# -------------------------------------------------------------
# Main page
# -------------------------------------------------------------
{
    # -----------------------------------------------------------
    # "Global" variables for main page
    # -----------------------------------------------------------
    my $bgcolor;            
    my $bgimage;
    my $rect_menu;
    my $itemheight;
    my $actions_key_press;
    my $title;
    my $x_title;
    my $y_title;
    my %font;
    # -----------------------------------------------------------
    sub config_page_main {
    	# -------------------------------------------------------
	# Load widget config: fill %BACKGROUND, %TITLE, %MENU
    	# -------------------------------------------------------
	do $FILES{page_main};
	# -------------------------------------------------------
	# Create background color and background image
	# -------------------------------------------------------
	$bgcolor = new SDL::Color( -r => $BACKGROUND{bgcolor}->[0], 
	                           -g => $BACKGROUND{bgcolor}->[1],
				   -b => $BACKGROUND{bgcolor}->[2],
	);
	$bgimage = new SDL::Surface(-name=> $BACKGROUND{image});
	# -------------------------------------------------------
	# Coordinates for title
	# -------------------------------------------------------
	$x_title = $TITLE{x};
	$y_title = $TITLE{y};
	$title   = $TITLE{text};
	# -------------------------------------------------------
	# Create rectangle for menu
	# -------------------------------------------------------
	$rect_menu = new SDL::Rect (
	    -width  => $MENU{width},
	    -height => $MENU{height},
	    -x      => $MENU{x},
	    -y      => $MENU{y},
	);	
	$itemheight = $MENU{itemheight};	
	# -------------------------------------------------------
	# Define fonts for title, item and highlight item
	# -------------------------------------------------------
	$font{title} =  new SDL::TTFont (  
		-name   => $TITLE{font},
		-size   => $TITLE{fontsize},
		-fg     => new SDL::Color ( 
			    -r => $TITLE{fgfontcolor}[0],
    			    -g => $TITLE{fgfontcolor}[1],
			    -b => $TITLE{fgfontcolor}[2], 
		),
		-bg     => new SDL::Color ( 
			    -r => $TITLE{bgfontcolor}[0],
    		    	    -g => $TITLE{bgfontcolor}[1],
			    -b => $TITLE{bgfontcolor}[2], 
		),
	);
	$font{item} =  new SDL::TTFont (  
		-name   => $MENU{font},
		-size   => $MENU{fontsize},
		-fg     => new SDL::Color ( 
			    -r => $MENU{fgfontcolor}[0],
    			    -g => $MENU{fgfontcolor}[1],
			    -b => $MENU{fgfontcolor}[2], 
		),
		-bg     => new SDL::Color ( 
			    -r => $MENU{bgfontcolor}[0],
    		    	    -g => $MENU{bgfontcolor}[1],
			    -b => $MENU{bgfontcolor}[2], 
		),
	);
	$font{hitem} =  new SDL::TTFont (  
		-name   => $MENU{font},
		-size   => $MENU{fontsize},
		-fg     => new SDL::Color ( 
			    -r => $MENU{hlfontcolor}[0],
    			    -g => $MENU{hlfontcolor}[1],
			    -b => $MENU{hlfontcolor}[2], 
		),
		-bg     => new SDL::Color ( 
			    -r => $MENU{bgfontcolor}[0],
    		    	    -g => $MENU{bgfontcolor}[1],
			    -b => $MENU{bgfontcolor}[2], 
		),
	);
	# -------------------------------------------------------
	# Define actions for key press
	# -------------------------------------------------------
	$actions_key_press = {
	    'down'   =>  \&next_category,	
	    'up'     =>  \&prev_category,
	    'left'   =>  \&prev_category,	
	    'right'  =>  \&next_category,
	    'return' =>  \&draw_page_games,
	};
	# -------------------------------------------------------
	# Load categories: fill @CATEGORIES
	# -------------------------------------------------------
	do $FILES{categories};	
	# -------------------------------------------------------
	# Set initial category
	# -------------------------------------------------------
	$category = 0;  
	# -------------------------------------------------------
    }    

    sub draw_page_main {
	# -------------------------------------------------------
	# Configure page the first time
	# -------------------------------------------------------
        config_page_main() unless defined $bgcolor;
	# -------------------------------------------------------
	# Set actions for main loop
	# -------------------------------------------------------
	$ACTIONS = $actions_page_intro;
	# -------------------------------------------------------
	# Draw widgets on page
	# -------------------------------------------------------
	# Fill background
	# ---------------
	$APP     -> fill(0,$bgcolor);
        $bgimage -> blit(0,$APP,0);    
	# ---------------
	# Put title
	# ---------------
	$font{title} -> print ($APP,$x_title,$y_title,$title);
	# ---------------
	# Draw items on menu    
	# ---------------
	my $x = $rect_menu->x();
	my $y = $rect_menu->y();
	foreach (@CATEGORIES) {
	    $font{item} -> print ($APP,$x,$y,$_->{title});
	    $y += $itemheight;
	}	
	# ---------------    
	# Highlight $category item ( = 0 first time)
	# ---------------
	$font{hitem} -> print ($APP,
	                      $rect_menu->x(),
	                      $rect_menu->y()+$category*$itemheight,
			      $CATEGORIES[$category]->{title}
	);
	# -------------------------------------------------------
	# Set initial game 
	# -------------------------------------------------------
	$widget = 0;
	# -------------------------------------------------------
	# Performs a full update of the application surface 
	# -------------------------------------------------------
	$APP -> sync();  
	# -------------------------------------------------------
    }   

    # -----------------------------------------------------------
    # Subrutines for actions 
    # -----------------------------------------------------------
    sub process_mousemotion_page_intro {
	my $event = shift;
	my $x = $event-> motion_x();
	my $y = $event-> motion_y();
	if ( $x > $rect_menu->x() and $y > $rect_menu->y() and 
	     $x < $rect_menu->x() + $rect_menu->width()    and 
	     $y < $rect_menu->y() + $rect_menu->height() ) {
	    my $old = $category;
	    my $new = int ( ($y-$rect_menu->y())/$itemheight );
	    if ( $new < @CATEGORIES and $new != $old ) {
		swap_categories($old,$new);
		$category = $new;
	    }
        }
    }

    sub process_mousebuttondown_page_intro {
	my $event = shift;
	if ( $event -> button() == 1 ) {
	    my $x = $event-> motion_x();
	    my $y = $event-> motion_y();
	    if ( $x > $rect_menu->x() and $y > $rect_menu->y() and 
	        $x < $rect_menu->x() + $rect_menu->width()    and 
	        $y < $rect_menu->y() + $rect_menu->height() ) {
		my $new = int ( ($y-$rect_menu->y())/$itemheight );
		if ( $new < @CATEGORIES ) {
		    $category = $new;
		    draw_page_games();
		}
    	    }
	}
    }	

    sub process_keydown_page_intro {
	my $event = shift;
	my $key = $event->key_name();
	if ( exists($actions_key_press->{$key}) && ref( $actions_key_press->{$key} ) eq "CODE" ) {
	    $actions_key_press->{$key}->();
	}	    
    }

    sub next_category {
	my $old = $category;
	my $new = $category+1;
	if ( $new < @CATEGORIES ) {
	    swap_categories($old,$new);
	    $category = $new;
	}
	else {
	    swap_categories($old,0);
	    $category = 0;
	}	    
    }

    sub prev_category {
	my $old = $category;
	my $new = $category-1;
	if ( $new >= 0 ) {
	    swap_categories($old,$new);
	    $category = $new;
	}
	else {
	    swap_categories($old, $#CATEGORIES);
	    $category = scalar $#CATEGORIES;
	}
    }
    # -----------------------------------------------------------
    # Highlight category selected
    # -----------------------------------------------------------
    sub swap_categories {
	my ($old,$new) = @_;
	$font{item}  -> print ($APP,$rect_menu->x(),$rect_menu->y()+$old*$itemheight,$CATEGORIES[$old]->{title});
	$font{hitem} -> print ($APP,$rect_menu->x(),$rect_menu->y()+$new*$itemheight,$CATEGORIES[$new]->{title});	
	$APP-> update($rect_menu);
    }

} 
# -------------------------------------------------------------
# Games page
# -------------------------------------------------------------
{
    my $bgcolor;
    my $bgimg;
    my $qimg_on;
    my $qimg_off;
    my $x_title;
    my $y_title;
    my @box;
    my @grid;
    my @delta;
    my @rectangulos;
    my @imgs_on;
    my @imgs_off;
    my $rect_info;
    my $rect_quit;
    my $actions_key_press;
    my %font;  # keys: title, info, subinfo
    my $game;
    
    sub config_page_games {
    	# -------------------------------------------------------
	# Load widget config: fill %TITLE, %MENU, %INFO, %BACK
    	# -------------------------------------------------------
	do $FILES{page_games};
    	# -------------------------------------------------------    
	# Coordinates for title
    	# -------------------------------------------------------    	
	$x_title = $TITLE{x};
	$y_title = $TITLE{y};
    	# -------------------------------------------------------    
	# Create space for games rectangles
    	# -------------------------------------------------------    
	@box   = ($MENU{x},$MENU{y},$MENU{x}+$MENU{width},$MENU{y}+$MENU{height});
	@grid  = ($MENU{columns}, $MENU{rows});
	@delta = (($box[2]-$box[0])/$grid[0],($box[3]-$box[1])/$grid[1]);	
    	# -------------------------------------------------------    
	# Create rectangle for display game info 
    	# -------------------------------------------------------    
	$rect_info = new SDL::Rect (
			-width  => $INFO{width}, 
			-height => $INFO{height},
			-x      => $INFO{x},
    			-y      => $INFO{y},
	);
    	# -------------------------------------------------------    
	# Create rectangle for back main page and surfaces for
	# the images
    	# -------------------------------------------------------    
	$rect_quit = new SDL::Rect (
			-width  => $BACK{width}, 
			-height => $BACK{height},                   
			-x      => $BACK{x},
    			-y      => $BACK{y},
	);
	$qimg_on  = new SDL::Surface(-name=> $BACK{image_on});
	$qimg_off = new SDL::Surface(-name=> $BACK{image_off});
    	# -------------------------------------------------------    
	# Define actions for key press
    	# -------------------------------------------------------    
	$actions_key_press = {
	    'backspace' => \&draw_page_main,
	    'return'    => \&run_widget,
	    'right'     => \&right_widget,
	    'left'      => \&left_widget,	
	    'down'      => \&down_widget,	
	    'up'        => \&up_widget,
	    'tab'       => \&tab_widget,
	};
    }

    sub config_this_page {
	# -------------------------------------------------------
	my $rcfile = shift;
	# -------------------------------------------------------
	# Load specific widget configuration: 
	# fill %BACKGROUND and %FONTS 
	# -------------------------------------------------------
	do $rcfile;
	# -------------------------------------------------------
	# Create background color and background image
	# -------------------------------------------------------
	$bgcolor = new SDL::Color( -r => $BACKGROUND{bgcolor}->[0], 
	                           -g => $BACKGROUND{bgcolor}->[1],
				   -b => $BACKGROUND{bgcolor}->[2],
	);
	$bgimg = new SDL::Surface( -name=> $BACKGROUND{image});
	# -------------------------------------------------------
	# Create fonts
	# -------------------------------------------------------
	foreach (qw/title info subinfo/) {
    	    $font{$_} = new SDL::TTFont (  
			    -name   => $FONTS{$_}->{font},
			    -size   => $FONTS{$_}->{size},
			    -fg     => new SDL::Color ( 
					-r => $FONTS{$_}->{fgcolor}[0],
    					-g => $FONTS{$_}->{fgcolor}[1],
					-b => $FONTS{$_}->{fgcolor}[2],
			    ),
			    -bg     => new SDL::Color ( 
					-r => $FONTS{$_}->{bgcolor}[0],
    					-g => $FONTS{$_}->{bgcolor}[1],
					-b => $FONTS{$_}->{bgcolor}[2],
			    ),
	    );
	}
	# -------------------------------------------------------
	# Set initial game to initial widget
	# -------------------------------------------------------
	$game = $widget;
    }

    sub draw_page_games {
    	# -------------------------------------------------------    
	config_page_games() unless @box;
        config_this_page($CATEGORIES[$category]->{rcfile}); 
    	# -------------------------------------------------------        
	# Load games on @GAMES
    	# -------------------------------------------------------    
	do $CATEGORIES[$category]->{gamefile};
	# -------------------------------------------------------
	# Draw widgets on page
	# -------------------------------------------------------
	# Fill background
	# ---------------
	$APP   -> fill(0, $bgcolor);
    	$bgimg -> blit(0,$APP,0);
	# -----------------
	# Create rectangles, load images and fill games
    	# -----------------
	@rectangulos  = ();   # reset! 
	@imgs_on      = ();
	@imgs_off     = ();
	my $count = 0;
	MAIN_LOOP:
        foreach my $j (0..($grid[1]-1)) {
	    foreach my $i (0..($grid[0]-1)) {
		last MAIN_LOOP if $count == @GAMES;
		push  @rectangulos , new SDL::Rect (
		    -width  => $delta[0], 
		    -height => $delta[1],                   
		    -x      => $box[0]+$i*$delta[0], 
    		    -y      => $box[1]+$j*$delta[1], 
		); 
		push @imgs_off,  new SDL::Surface(-name=> $GAMES[$count]{unselected}); 
		push @imgs_on ,  new SDL::Surface(-name=> $GAMES[$count]{selected}); 
		$imgs_off[$count] -> blit(0,$APP,$rectangulos[$count]);
		$count++;
	    }
	}    
    	# --------------
	# Add rect_quit and images on bottom arrays
    	# --------------
	push @rectangulos, $rect_quit;
	push @imgs_off, $qimg_off;
	push @imgs_on,  $qimg_on;
    	# ---------------
	# Put back image on rect_quit
    	# ---------------
	$imgs_off[@GAMES] -> blit(0,$APP,$rectangulos[@GAMES]);
    	# ---------------
	# Put the title
    	# ---------------
	$font{title} -> print($APP,$x_title,$y_title,$CATEGORIES[$category]->{title});
    	# ---------------
	# Highlight game (0 first time)
    	# ---------------
	$imgs_on[$widget] -> blit(0,$APP,$rectangulos[$widget]);
    	# ---------------
	# Put game info on rect_info
    	# ---------------
	$APP   -> fill($rect_info,$bgcolor);
    	$bgimg -> blit($rect_info,$APP,$rect_info); 
	$font{info} -> print($APP,$rect_info->x(),$rect_info->y(),$GAMES[$widget]->{title});
	if ( exists $GAMES[$widget]->{subtitle} ) {
	    $font{subinfo} -> print($APP,$rect_info->x(),$rect_info->y()+$font{info}->height(),$GAMES[$widget]->{subtitle});
	}	  
    	# -------------------------------------------------------    
	# Define actions on this page
    	# -------------------------------------------------------    
	$ACTIONS = $actions_page_games;
	# -------------------------------------------------------
	# Performs a full update of the application surface, 
	# -------------------------------------------------------
	$APP -> sync();  
    	# -------------------------------------------------------    
    }

    # -----------------------------------------------------------
    # Subrutines for actions 
    # -----------------------------------------------------------
    sub process_mousemotion_page_games {
	my $event = shift;
	my $x = $event-> motion_x();
	my $y = $event-> motion_y();
	if ( $x > $box[0] and $y > $box[1] and $x < $box[2] and $y < $box[3]) {
	    my $old = $widget;
	    my $new = int ( ($x-$box[0])/$delta[0] ) + int ( ($y-$box[1])/$delta[1] )*$grid[0];
	    if ( $new < @GAMES and $new != $old ) {
		swap_rectangulos($old,$new);
		redraw_widget_info();
	    }
        }
	if ( $x > $rectangulos[@GAMES]->x() and $y > $rectangulos[@GAMES]->y() and
	     $x < $rectangulos[@GAMES]->x()+$rectangulos[@GAMES]->width() and
	     $y < $rectangulos[@GAMES]->y()+$rectangulos[@GAMES]->height() ) {
	     my $old = $widget;
	     my $new = scalar @GAMES;
	     if ( $old != $new ) {
		 swap_rectangulos($old,$new);
	         redraw_widget_info();
	     }		
	}
    }

    sub process_mousebuttondown_page_games {
	my $event = shift;
	if ( $event -> button() == 1 ) {
	    my $x = $event-> motion_x();
	    my $y = $event-> motion_y();
	    if ( $x > $box[0] and $y > $box[1] and $x < $box[2] and $y < $box[3]) {
		my $new = int ( ($x-$box[0])/$delta[0] ) + int ( ($y-$box[1])/$delta[1] )*$grid[0];
		if ( $new < @GAMES ) {
		    $widget = $new;
		    run_widget();
		}
	    }
	    if ( $x > $rectangulos[@GAMES]->x() and $y > $rectangulos[@GAMES]->y() and
	         $x < $rectangulos[@GAMES]->x()+$rectangulos[@GAMES]->width() and
		 $y < $rectangulos[@GAMES]->y()+$rectangulos[@GAMES]->height() ) {
		 $widget = scalar @GAMES;
		 run_widget();
	    }
	}
    }    

    sub process_keydown_page_games {
	my $event = shift;
	my $key = $event->key_name();
	if ( exists($actions_key_press->{$key}) && ref( $actions_key_press->{$key} ) eq "CODE" ) {
	    $actions_key_press->{$key}->();
	}	    
    }

    sub right_widget {
	my $old = $widget;
	my $new = $widget + 1;
	$new = 0 if $new == @GAMES + 1;
	swap_rectangulos($old,$new);
	redraw_widget_info();
    }

    sub left_widget {
    	my $old = $widget;
	my $new = $widget - 1;
	$new = scalar @GAMES if $new < 0;
	swap_rectangulos($old,$new);
	redraw_widget_info();
    }

    sub down_widget {
	return unless $widget < @GAMES;
	my $old = $widget;
	my $new = $widget + $grid[0];
	if ( $new < @GAMES ) {
	    swap_rectangulos($old,$new);
	    redraw_widget_info();
	}
    }

    sub up_widget {
	return unless $widget < @GAMES;
	my $old = $widget;
	my $new = $widget - $grid[0];
	if ( $new >= 0 ) {
	    swap_rectangulos($old,$new);
	    redraw_widget_info();
	}
    }

    sub run_widget {
	if ($widget < @GAMES ) {
    	    warn "Running game: $widget on file $GAMES[$widget]->{play}\n";
	    do $GAMES[$widget]->{play};
	}	  
	else { 
	    draw_page_main(); # back main page
	}  
    }
    
    sub swap_rectangulos {
	my ($old, $new)  = @_;
	$imgs_off[$old] -> blit(0,$APP,$rectangulos[$old]);
	$imgs_on[$new]  -> blit(0,$APP,$rectangulos[$new]);
	$APP -> update($rectangulos[$old]);
	$APP -> update($rectangulos[$new]);
	$widget = $new;
	if ($new < @GAMES) {
	    $game = $new;
	}	    

    }

    sub redraw_widget_info {
    	$APP   -> fill($rect_info,$bgcolor);
    	$bgimg -> blit($rect_info,$APP,$rect_info); 
	if ( $widget < @GAMES ) {	
	    $font{info} -> print($APP,$rect_info->x(),$rect_info->y(),$GAMES[$widget]->{title});
	    if ( exists $GAMES[$widget]->{subtitle} ) {
		$font{subinfo} -> print($APP,$rect_info->x(),$rect_info->y()+$font{info}->height(),$GAMES[$widget]->{subtitle});
	    }	    
	}
	else {
	    $font{info} -> print($APP,$rect_info->x(),$rect_info->y()," ");
	    $font{subinfo} -> print($APP,$rect_info->x(),$rect_info->y()+$font{info}->height(),"Volver al men principal");
	}	    
	$APP->update($rect_info);
    }

    sub tab_widget {
	if ( $widget < @GAMES ) {
	    swap_rectangulos($widget, scalar @GAMES);
	} 
	else {
	    swap_rectangulos($widget,$game);
	}	    
	redraw_widget_info();
    }

}
